@@ -48,7 +48,9 @@ DlgAbout::DlgAbout(QWidget* parent) : QDialog(parent), Ui::DlgAboutDlg() {
             << "Nicu Badescu"
             << "Uwe Klotz"
             << "Be"
-            << "S&eacute;bastien Blaisot";
+            << "S&eacute;bastien Blaisot"
+            << "ronso0"
+            << "Jan Holthuis";
 
     // This list should contains all contributors committed 
     // code to the Mixxx core within the past two years. 
@@ -74,6 +76,7 @@ DlgAbout::DlgAbout(QWidget* parent) : QDialog(parent), Ui::DlgAboutDlg() {
             << "Kilian Feess"
             << "Conner Phillips"
             << "Daniel Poelzleithner"
+            << "St&eacute;phane Lepin"
             << "Artyom Lyan"
             << "Johan Lasperas"
             << "Olaf Hering"
@@ -82,7 +85,6 @@ DlgAbout::DlgAbout(QWidget* parent) : QDialog(parent), Ui::DlgAboutDlg() {
             << "Kshitij Gupta"
             << "Thomas Jarosch"
             << "Matthew Nicholson"
-            << "ronso0"
             << "Jamie Gifford"
             << "luzpaz"
             << "Sebastian Reu&szlig;e"
@@ -90,7 +92,8 @@ DlgAbout::DlgAbout(QWidget* parent) : QDialog(parent), Ui::DlgAboutDlg() {
             << "beenisss"
             << "Bernd Binder"
             << "Nikolaus Einhauser"
-            << "Philip Gottschling"; 
+            << "Philip Gottschling"
+            << "Cristiano Lacerda"; 
 
     QStringList specialThanks;
     specialThanks
