@@ -371,34 +371,36 @@ void CrateFeature::slotRenameCrate() {
     if (readLastRightClickedCrate(&crate)) {
         const QString oldName = crate.getName();
         crate.resetName();
-        while (!crate.hasName()) {
+        for (;;) {
             bool ok = false;
-            crate.parseName(
+            auto newName =
                     QInputDialog::getText(
                             nullptr,
                             tr("Rename Crate"),
                             tr("Enter new name for crate:"),
                             QLineEdit::Normal,
                             oldName,
-                            &ok));
-            if (!ok || (crate.getName() == oldName)) {
+                            &ok).trimmed();
+            if (!ok || newName.isEmpty()) {
                 return;
             }
-            if (!crate.hasName()) {
+            if (newName.isEmpty()) {
                 QMessageBox::warning(
                         nullptr,
                         tr("Renaming Crate Failed"),
                         tr("A crate cannot have a blank name."));
                 continue;
             }
-            if (m_pTrackCollection->crates().readCrateByName(crate.getName())) {
+            if (m_pTrackCollection->crates().readCrateByName(newName)) {
                 QMessageBox::warning(
                         nullptr,
                         tr("Renaming Crate Failed"),
                         tr("A crate by that name already exists."));
-                crate.resetName();
                 continue;
             }
+            crate.setName(std::move(newName));
+            DEBUG_ASSERT(crate.hasName());
+            break;
         }
 
         if (!m_pTrackCollection->updateCrate(crate)) {
@@ -594,15 +596,16 @@ void CrateFeature::slotCreateImportCrate() {
         // Get a valid name
         QString baseName = fileName.baseName();
         for (int i = 0;; ++i) {
-            QString name = baseName;
+            auto name = baseName;
             if (i > 0) {
                 name += QString(" %1").arg(i);
             }
-
-            if (crate.parseName(name)) {
-                DEBUG_ASSERT(crate.hasName());
-                if (!m_pTrackCollection->crates().readCrateByName(crate.getName())) {
+            name = name.trimmed();
+            if (!name.isEmpty()) {
+                if (!m_pTrackCollection->crates().readCrateByName(name)) {
                     // unused crate name found
+                    crate.setName(std::move(name));
+                    DEBUG_ASSERT(crate.hasName());
                     break; // terminate loop
                 }
             }
@@ -700,21 +703,10 @@ void CrateFeature::slotExportPlaylist() {
             QModelIndex index = m_crateTableModel.index(i, 0);
             playlist_items << m_crateTableModel.getTrackLocation(index);
         }
-
-        if (file_location.endsWith(".pls", Qt::CaseInsensitive)) {
-            ParserPls::writePLSFile(file_location, playlist_items, useRelativePath);
-        } else if (file_location.endsWith(".m3u8", Qt::CaseInsensitive)) {
-            ParserM3u::writeM3U8File(file_location, playlist_items, useRelativePath);
-        } else {
-            //default export to M3U if file extension is missing
-            if(!file_location.endsWith(".m3u", Qt::CaseInsensitive))
-            {
-                qDebug() << "Crate export: No valid file extension specified. Appending .m3u "
-                         << "and exporting to M3U.";
-                file_location.append(".m3u");
-            }
-            ParserM3u::writeM3UFile(file_location, playlist_items, useRelativePath);
-        }
+        exportPlaylistItemsIntoFile(
+                file_location,
+                playlist_items,
+                useRelativePath);
     }
 }
 
