@@ -135,7 +135,7 @@ QString ParserPls::getFilepath(QTextStream *stream, QString basepath) {
     return 0;
 }
 
-bool ParserPls::writePLSFile(const QString &file_str, QList<QString> &items, bool useRelativePath)
+bool ParserPls::writePLSFile(const QString &file_str, const QList<QString> &items, bool useRelativePath)
 {
     QFile file(file_str);
     if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
