@@ -26,7 +26,7 @@ class ParserPls : public Parser {
     /**Can be called to parse a pls file**/
     QList<QString> parse(QString);
     //Playlist Export
-    static bool writePLSFile(const QString &file, QList<QString> &items, bool useRelativePath);
+    static bool writePLSFile(const QString &file, const QList<QString> &items, bool useRelativePath);
 
   private:
     /**Returns the Number of entries in the pls file**/
