@@ -11,9 +11,11 @@
 #include "util/trace.h"
 #include "util/file.h"
 #include "util/timer.h"
+#include "util/performancetimer.h"
 #include "library/scanner/scannerutil.h"
 #include "util/db/dbconnectionpooler.h"
 #include "util/db/dbconnectionpooled.h"
+#include "util/db/fwdsqlquery.h"
 
 namespace {
 
@@ -24,6 +26,21 @@ mixxx::Logger kLogger("LibraryScanner");
 
 QAtomicInt s_instanceCounter(0);
 
+const QString kDeleteOrphanedLibraryScannerDirectories =
+        "delete from LibraryHashes where hash <> 0 and directory_path not in (select directory from track_locations)";
+
+// Returns the number of affected rows or -1 on error
+int execCleanupQuery(QSqlDatabase database, const QString& statement) {
+    FwdSqlQuery query(database, statement);
+    VERIFY_OR_DEBUG_ASSERT(query.isPrepared()) {
+        return -1;
+    }
+    if (!query.execPrepared()) {
+        return -1;
+    }
+    return query.numRowsAffected();
+}
+
 } // anonymous namespace
 
 LibraryScanner::LibraryScanner(
@@ -103,6 +120,26 @@ void LibraryScanner::run() {
             return;
         }
 
+        // Clean up the database and fix inconsistencies from previous runs.
+        // See also: https://bugs.launchpad.net/mixxx/+bug/1846945
+        {
+            kLogger.info() << "Cleaning up database...";
+            PerformanceTimer timer;
+            timer.start();
+            auto numRows = execCleanupQuery(dbConnection,
+                    kDeleteOrphanedLibraryScannerDirectories);
+            if (numRows < 0) {
+                kLogger.warning()
+                        << "Failed to delete orphaned directory hashes";
+            } else if (numRows > 0) {
+                kLogger.info()
+                        << "Deleted" << numRows << "orphaned directory hashes)";
+            }
+            kLogger.info()
+                    << "Finished database cleanup:"
+                    << timer.elapsed().debugMillisWithUnit();
+        }
+
         m_libraryHashDao.initialize(dbConnection);
         m_cueDao.initialize(dbConnection);
         m_trackDao.initialize(dbConnection);
