@@ -43,8 +43,6 @@ BaseTrackPlayerImpl::BaseTrackPlayerImpl(QObject* pParent,
                                 pEffectsManager, defaultOrientation);
 
     m_pInputConfigured = std::make_unique<ControlProxy>(group, "input_configured", this);
-    m_pPassthroughEnabled = std::make_unique<ControlProxy>(group, "passthrough", this);
-    m_pPassthroughEnabled->connectValueChanged(SLOT(slotPassthroughEnabled(double)));
 #ifdef __VINYLCONTROL__
     m_pVinylControlEnabled = std::make_unique<ControlProxy>(group, "vinylcontrol_enabled", this);
     m_pVinylControlEnabled->connectValueChanged(SLOT(slotVinylControlEnabled(double)));
@@ -408,18 +406,6 @@ void BaseTrackPlayerImpl::setupEqControls() {
     m_pPitchAdjust = std::make_unique<ControlProxy>(group, "pitch_adjust", this);
 }
 
-void BaseTrackPlayerImpl::slotPassthroughEnabled(double v) {
-    bool configured = m_pInputConfigured->toBool();
-    bool passthrough = v > 0.0;
-
-    // Warn the user if they try to enable passthrough on a player with no
-    // configured input.
-    if (!configured && passthrough) {
-        m_pPassthroughEnabled->set(0.0);
-        emit(noPassthroughInputConfigured());
-    }
-}
-
 void BaseTrackPlayerImpl::slotVinylControlEnabled(double v) {
 #ifdef __VINYLCONTROL__
     bool configured = m_pInputConfigured->toBool();
