@@ -7,9 +7,10 @@
 
 #include "preferences/usersettings.h"
 #include "track/track.h"
+#include "widget/trackdroptarget.h"
 #include "widget/wlabel.h"
 
-class WTrackText : public WLabel {
+class WTrackText : public WLabel, public TrackDropTarget {
     Q_OBJECT
   public:
     WTrackText(const char* group, UserSettingsPointer pConfig, QWidget *pParent);
